// Copyright (c) 2017, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library jaguar_oauth2;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_session/jaguar_session.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_oauth/jaguar_oauth.dart';
import 'package:oauth2/oauth2.dart' as oauth2;
import 'package:jaguar_facebook_client/jaguar_facebook_client.dart' as fb;

abstract class FacebookUserModel implements UserModel {
  String get fbId;
}

abstract class FacebookModelManager {
  Future<FacebookUserModel> fetchModelByFbId(String authenticationId);
}

class WrapFacebookAuth implements RouteWrapper<FacebookAuth> {
  final FacebookModelManager manager;

  final String authorizationIdKey;

  final String id;

  final Map<Symbol, MakeParam> makeParams;

  const WrapFacebookAuth(this.manager,
      {this.authorizationIdKey: 'id', this.id, this.makeParams});

  FacebookAuth createInterceptor() => new FacebookAuth(this.manager,
      authorizationIdKey: this.authorizationIdKey);
}

class FacebookAuth extends Interceptor {
  final FacebookModelManager manager;

  final String authorizationIdKey;

  final bool manageSession;

  @Input(SessionInterceptor)
  FacebookAuth(this.manager,
      {this.authorizationIdKey: 'id', this.manageSession: true});

  Future<FacebookUserModel> pre(@Input(OAuth2Authorized) oauth2.Client client,
      @Input(SessionInterceptor) SessionManager sessionManager) async {
    final graph = new fb.GraphApi(client);
    final fields = new fb.UserFieldSelector()
      ..addBirthday()
      ..addAbout()
      ..addEmail()
      ..addName();
    final resp = await graph.getMe(fields: fields);

    final FacebookUserModel model = await manager.fetchModelByFbId(resp.id);

    if (model == null) {
      throw new UnAuthorizedError();
    }

    if (manageSession) {
      sessionManager.updateSession({authorizationIdKey: model.authorizationId});
    }

    return model;
  }
}
